# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from collections import defaultdict

import six

from .base_account import BaseAccount
from ...events import EVENT
from ...environment import Environment
from ...utils.logger import user_system_log
from ...utils.i18n import gettext as _
from ...const import SIDE, ACCOUNT_TYPE


class StockAccount(BaseAccount):
    def __init__(self, total_cash, positions, backward_trade_set=set(), dividend_receivable=None, register_event=True):
        super(StockAccount, self).__init__(total_cash, positions, backward_trade_set, register_event)
        self._dividend_receivable = dividend_receivable if dividend_receivable else {}

    def register_event(self):
        event_bus = Environment.get_instance().event_bus
        event_bus.add_listener(EVENT.TRADE, self._on_trade)
        event_bus.add_listener(EVENT.ORDER_PENDING_NEW, self._on_order_pending_new)
        event_bus.add_listener(EVENT.ORDER_CREATION_REJECT, self._on_order_unsolicited_update)
        event_bus.add_listener(EVENT.ORDER_UNSOLICITED_UPDATE, self._on_order_unsolicited_update)
        event_bus.add_listener(EVENT.ORDER_CANCELLATION_PASS, self._on_order_unsolicited_update)
        event_bus.add_listener(EVENT.PRE_BEFORE_TRADING, self._before_trading)
        event_bus.add_listener(EVENT.SETTLEMENT, self._on_settlement)

    def get_state(self):
        return {
            'positions': {
                order_book_id: position.get_state()
                for order_book_id, position in six.iteritems(self._positions)
            },
            'frozen_cash': self._frozen_cash,
            'total_cash': self._total_cash,
            'backward_trade_set': list(self._backward_trade_set),
            'dividend_receivable': self._dividend_receivable,
        }

    def set_state(self, state):
        self._frozen_cash = state['frozen_cash']
        self._total_cash = state['total_cash']
        self._backward_trade_set = set(state['backward_trade_set'])
        self._dividend_receivable = state['dividend_receivable']
        self._positions.clear()
        for order_book_id, v in six.iteritems(state['positions']):
            position = self._positions.get_or_create(order_book_id)
            position.set_state(v)

    def fast_forward(self, orders, trades=list()):
        # 计算 Positions
        for trade in trades:
            if trade.exec_id in self._backward_trade_set:
                continue
            self._apply_trade(trade)
        # 计算 Frozen Cash
        self._frozen_cash = 0
        frozen_quantity = defaultdict(int)
        for o in orders:
            if o.is_final():
                continue
            if o.side == SIDE.BUY:
                self._frozen_cash += o.frozen_price * o.unfilled_quantity
            else:
                frozen_quantity[o.order_book_id] += o.unfilled_quantity
        for order_book_id, position in six.iteritems(self._positions):
            position.reset_frozen(frozen_quantity[order_book_id])

    def _on_trade(self, event):
        if event.account != self:
            return
        self._apply_trade(event.trade)

    def _apply_trade(self, trade):
        if trade.exec_id in self._backward_trade_set:
            return

        position = self._positions.get_or_create(trade.order_book_id)
        position.apply_trade(trade)
        self._total_cash -= trade.transaction_cost
        if trade.side == SIDE.BUY:
            self._total_cash -= trade.last_quantity * trade.last_price
            self._frozen_cash -= trade.frozen_price * trade.last_quantity
        else:
            self._total_cash += trade.last_price * trade.last_quantity
        self._backward_trade_set.add(trade.exec_id)

    def _on_order_pending_new(self, event):
        if event.account != self:
            return
        order = event.order
        position = self._positions.get_or_create(order.order_book_id)
        position.on_order_pending_new_(order)
        if order.side == SIDE.BUY:
            order_value = order.frozen_price * order.quantity
            self._frozen_cash += order_value

    def _on_order_unsolicited_update(self, event):
        if event.account != self:
            return

        order = event.order
        position = self._positions.get_or_create(order.order_book_id)
        position.on_order_cancel_(order)
        if order.side == SIDE.BUY:
            unfilled_value = order.unfilled_quantity * order.frozen_price
            self._frozen_cash -= unfilled_value

    def _before_trading(self, event):
        trading_date = Environment.get_instance().trading_dt.date()
        self._handle_dividend_payable(trading_date)
        if Environment.get_instance().config.base.handle_split:
            self._handle_split(trading_date)

    def _on_settlement(self, event):
        for position in list(self._positions.values()):
            order_book_id = position.order_book_id
            if position.is_de_listed() and position.quantity != 0:
                if Environment.get_instance().config.validator.cash_return_by_stock_delisted:
                    self._total_cash += position.market_value
                user_system_log.warn(
                    _(u"{order_book_id} is expired, close all positions by system").format(order_book_id=order_book_id)
                )
                self._positions.pop(order_book_id, None)
            elif position.quantity == 0:
                self._positions.pop(order_book_id, None)
            else:
                position.apply_settlement()

        self._backward_trade_set.clear()
        self._handle_dividend_book_closure(event.trading_dt.date())

    @property
    def type(self):
        return ACCOUNT_TYPE.STOCK

    def _handle_dividend_payable(self, trading_date):
        to_be_removed = []
        for order_book_id, dividend in six.iteritems(self._dividend_receivable):
            if dividend['payable_date'] == trading_date:
                to_be_removed.append(order_book_id)
                self._total_cash += dividend['quantity'] * dividend['dividend_per_share']
        for order_book_id in to_be_removed:
            del self._dividend_receivable[order_book_id]

    def _handle_dividend_book_closure(self, trading_date):
        for order_book_id, position in six.iteritems(self._positions):
            dividend = Environment.get_instance().data_proxy.get_dividend_by_book_date(order_book_id, trading_date)
            if dividend is None:
                continue

            dividend_per_share = dividend['dividend_cash_before_tax'] / dividend['round_lot']
            self._dividend_receivable[order_book_id] = {
                'quantity': position.quantity,
                'dividend_per_share': dividend_per_share,
                'payable_date': dividend['payable_date'].date()
            }

    def _handle_split(self, trading_date):
        for order_book_id, position in six.iteritems(self._positions):
            split = Environment.get_instance().data_proxy.get_split_by_ex_date(order_book_id, trading_date)
            if split is None:
                continue
            ratio = split['split_coefficient_to'] / split['split_coefficient_from']
            position.split_(ratio)

    @property
    def total_value(self):
        return self.market_value + self._total_cash

    @property
    def dividend_receivable(self):
        """
        [float] 投资组合在分红现金收到账面之前的应收分红部分。具体细节在分红部分
        """
        return sum(d['quantity'] * d['dividend_per_share'] for d in six.itervalues(self._dividend_receivable))



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import inspect
import datetime
import six
import pandas as pd
from functools import wraps

from dateutil.parser import parse as parse_date

from .exception import RQInvalidArgument, RQTypeError
from ..model.instrument import Instrument
from ..environment import Environment
from ..const import INSTRUMENT_TYPE, RUN_TYPE
from ..utils import unwrapper, INST_TYPE_IN_STOCK_ACCOUNT
from ..utils.i18n import gettext as _
from ..utils.logger import user_system_log


main_contract_warning_flag = True
index_contract_warning_flag = True


class ArgumentChecker(object):
    def __init__(self, arg_name):
        self._arg_name = arg_name
        self._rules = []

    def is_instance_of(self, types):
        def check_is_instance_of(func_name, value):
            if not isinstance(value, types):
                raise RQInvalidArgument(
                    _('function {}: invalid {} argument, expect a value of type {}, got {} (type: {})').format(
                        func_name, self._arg_name, types, value, type(value)
                    ))

        self._rules.append(check_is_instance_of)
        return self

    def raise_not_valid_instrument_error(self, func_name, arg_name, value):
        raise RQInvalidArgument(
            _('function {}: invalid {} argument, expect a valid instrument/order_book_id/symbol, '
              'got {} (type: {})').format(
                func_name, self._arg_name, value, type(value)
            ))

    def raise_not_valid_stock_error(self, func_name, arg_name, value):
        raise RQInvalidArgument(
            _('function {}: invalid {} argument, expect a valid stock instrument/order_book_id/symbol, '
              'got {} (type: {})').format(
                func_name, self._arg_name, value, type(value)
            ))

    def raise_not_valid_future_error(self, func_name, arg_name, value):
        raise RQInvalidArgument(
            _('function {}: invalid {} argument, expect a valid future instrument/order_book_id/symbol, '
              'got {} (type: {})').format(
                func_name, self._arg_name, value, type(value)
            ))

    def _is_valid_instrument(self, func_name, value):
        config = Environment.get_instance().config
        global index_contract_warning_flag
        if isinstance(value, six.string_types):
            if config.base.run_type == RUN_TYPE.PAPER_TRADING:
                if "88" in value:
                    raise RQInvalidArgument(_(u"Main Future contracts[88] are not supported in paper trading."))
                if "99" in value:
                    raise RQInvalidArgument(_(u"Index Future contracts[99] are not supported in paper trading."))
            else:
                if "88" in value:
                    global main_contract_warning_flag
                    if main_contract_warning_flag:
                        main_contract_warning_flag = False
                        user_system_log.warn(_(u"Main Future contracts[88] are not supported in paper trading."))
                if "99" in value:
                    global index_contract_warning_flag
                    if index_contract_warning_flag:
                        index_contract_warning_flag = False
                        user_system_log.warn(_(u"Index Future contracts[99] are not supported in paper trading."))
            instrument = Environment.get_instance().get_instrument(value)
            if instrument is None:
                self.raise_not_valid_instrument_error(func_name, self._arg_name, value)
            return

        if isinstance(value, Instrument):
            return

        self.raise_not_valid_instrument_error(func_name, self._arg_name, value)

    def is_valid_instrument(self):
        self._rules.append(self._is_valid_instrument)
        return self

    def _is_valid_stock(self, func_name, value):
        if isinstance(value, six.string_types):
            instrument = Environment.get_instance().get_instrument(value)
            if instrument is None:
                self.raise_not_valid_instrument_error(func_name, self._arg_name, value)
            if instrument.enum_type not in INST_TYPE_IN_STOCK_ACCOUNT:
                self.raise_not_valid_stock_error(func_name, self._arg_name, value)
            return

        if isinstance(value, Instrument):
            if value.enum_type not in INST_TYPE_IN_STOCK_ACCOUNT:
                self.raise_not_valid_stock_error(func_name, self._arg_name, value)
            else:
                return

        self.raise_not_valid_instrument_error(func_name, self._arg_name, value)

    def is_valid_stock(self):
        self._rules.append(self._is_valid_stock)
        return self

    def _is_valid_future(self, func_name, value):
        if isinstance(value, six.string_types):
            instrument = Environment.get_instance().get_instrument(value)
            if instrument is None:
                self.raise_not_valid_instrument_error(func_name, self._arg_name, value)
            if instrument.enum_type != INSTRUMENT_TYPE.FUTURE:
                self.raise_not_valid_future_error(func_name, self._arg_name, value)
            return
        if isinstance(value, Instrument):
            if value.enum_type != INSTRUMENT_TYPE.FUTURE:
                self.raise_not_valid_future_error(func_name, self._arg_name, value)
            else:
                return
        self.raise_not_valid_instrument_error(func_name, self._arg_name, value)

    def is_valid_future(self):
        self._rules.append(self._is_valid_future)
        return self

    def _is_number(self, func_name, value):
        try:
            v = float(value)
        except ValueError:
            raise RQInvalidArgument(
                _('function {}: invalid {} argument, expect a number, got {} (type: {})').format(
                    func_name, self._arg_name, value, type(value))
            )

    def is_number(self):
        self._rules.append(self._is_number)
        return self

    def is_in(self, valid_values, ignore_none=True):
        def check_is_in(func_name, value):
            if ignore_none and value is None:
                return

            if value not in valid_values:
                raise RQInvalidArgument(
                    _('function {}: invalid {} argument, valid: {}, got {} (type: {})').format(
                        func_name, self._arg_name, repr(valid_values), value, type(value))
                )
            return

        self._rules.append(check_is_in)
        return self

    def are_valid_fields(self, valid_fields, ignore_none=True):
        valid_fields = set(valid_fields)

        def check_are_valid_fields(func_name, fields):
            if isinstance(fields, six.string_types):
                if fields not in valid_fields:
                    raise RQInvalidArgument(
                        _('function {}: invalid {} argument, valid fields are {}, got {} (type: {})').format(
                            func_name, self._arg_name, repr(valid_fields), fields, type(fields)
                        ))
                return

            if fields is None and ignore_none:
                return

            if isinstance(fields, list):
                invalid_fields = [field for field in fields if field not in valid_fields]
                if invalid_fields:
                    raise RQInvalidArgument(
                        _('function {}: invalid field {}, valid fields are {}, got {} (type: {})').format(
                            func_name, invalid_fields, repr(valid_fields), fields, type(fields)
                        ))

            raise RQInvalidArgument(
                _('function {}: invalid {} argument, expect a string or a list of string, got {} (type: {})').format(
                    func_name, self._arg_name, repr(fields), type(fields)
                ))
        self._rules.append(check_are_valid_fields)
        return self

    def _are_valid_instruments(self, func_name, values):
        if isinstance(values, (six.string_types, Instrument)):
            self._is_valid_instrument(func_name, values)
            return

        if isinstance(values, list):
            for v in values:
                self._is_valid_instrument(func_name, v)
            return

        raise RQInvalidArgument(
            _('function {}: invalid {} argument, expect a string or a list of string, got {} (type: {})').format(
                func_name, self._arg_name, repr(values), type(values)
            ))

    def are_valid_instruments(self):
        self._rules.append(self._are_valid_instruments)
        return self

    def is_valid_date(self, ignore_none=True):
        def check_is_valid_date(func_name, value):
            if ignore_none and value is None:
                return None
            if isinstance(value, (datetime.date, pd.Timestamp)):
                return
            if isinstance(value, six.string_types):
                try:
                    v = parse_date(value)
                except ValueError:
                    raise RQInvalidArgument(
                        _('function {}: invalid {} argument, expect a valid date, got {} (type: {})').format(
                            func_name, self._arg_name, value, type(value)
                        ))

            raise RQInvalidArgument(
                _('function {}: invalid {} argument, expect a valid date, got {} (type: {})').format(
                    func_name, self._arg_name, value, type(value)
                ))

        self._rules.append(check_is_valid_date)
        return self

    def is_greater_or_equal_than(self, low):
        def check_greater_or_equal_than(func_name, value):
            if value < low:
                raise RQInvalidArgument(
                    _('function {}: invalid {} argument, expect a value >= {}, got {} (type: {})').format(
                        func_name, self._arg_name, low, value, type(value)
                    ))
        self._rules.append(check_greater_or_equal_than)
        return self

    def is_greater_than(self, low):
        def check_greater_than(func_name, value):
            if value <= low:
                raise RQInvalidArgument(
                    _('function {}: invalid {} argument, expect a value > {}, got {} (type: {})').format(
                        func_name, self._arg_name, low, value, type(value)
                    ))
        self._rules.append(check_greater_than)
        return self

    def is_less_or_equal_than(self, high):
        def check_less_or_equal_than(func_name, value):
            if value > high:
                raise RQInvalidArgument(
                    _('function {}: invalid {} argument, expect a value <= {}, got {} (type: {})').format(
                        func_name, self._arg_name, high, value, type(value)
                    ))

        self._rules.append(check_less_or_equal_than)
        return self

    def is_less_than(self, high):
        def check_less_than(func_name, value):
            if value >= high:
                raise RQInvalidArgument(
                    _('function {}: invalid {} argument, expect a value < {}, got {} (type: {})').format(
                        func_name, self._arg_name, high, value, type(value)
                    ))

        self._rules.append(check_less_than)
        return self

    def _is_valid_interval(self, func_name, value):
        valid = isinstance(value, six.string_types) and value[-1] in {'d', 'm', 'q', 'y'}
        if valid:
            try:
                valid = int(value[:-1]) > 0
            except ValueError:
                valid = False

        if not valid:
            raise RQInvalidArgument(
                _(u"function {}: invalid {} argument, interval should be in form of '1d', '3m', '4q', '2y', "
                  "got {} (type: {})").format(
                    func_name, self.arg_name, value, type(value)
                ))

    def is_valid_interval(self):
        self._rules.append(self._is_valid_interval)
        return self

    def _are_valid_query_entities(self, func_name, entities):
        from sqlalchemy.orm.attributes import InstrumentedAttribute
        for e in entities:
            if not isinstance(e, InstrumentedAttribute):
                raise RQInvalidArgument(
                    _(u"function {}: invalid {} argument, should be entity like "
                      "Fundamentals.balance_sheet.total_equity, got {} (type: {})").format(
                        func_name, self.arg_name, e, type(e)
                    ))

    def are_valid_query_entities(self):
        self._rules.append(self._are_valid_query_entities)
        return self

    def _is_valid_frequency(self, func_name, value):
        valid = isinstance(value, six.string_types) and value[-1] in ('d', 'm')
        if valid:
            try:
                valid = int(value[:-1]) > 0
            except ValueError:
                valid = False

        if not valid:
            raise RQInvalidArgument(
                _(u"function {}: invalid {} argument, frequency should be in form of "
                  "'1m', '5m', '1d', got {} (type: {})").format(
                    func_name, self.arg_name, value, type(value)
                ))

    def is_valid_frequency(self):
        self._rules.append(self._is_valid_frequency)
        return self

    def verify(self, func_name, value):
        for r in self._rules:
            r(func_name, value)

    @property
    def arg_name(self):
        return self._arg_name


def verify_that(arg_name):
    return ArgumentChecker(arg_name)


def apply_rules(*rules):
    def decorator(func):
        @wraps(func)
        def api_rule_check_wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except RQInvalidArgument:
                raise
            except Exception:
                exc_info = sys.exc_info()
                t, v, tb = exc_info

                try:
                    call_args = inspect.getcallargs(unwrapper(func), *args, **kwargs)
                except TypeError as e:
                    raise RQTypeError(*e.args).with_traceback(tb)

                try:
                    for r in rules:
                        r.verify(func.__name__, call_args[r.arg_name])
                except RQInvalidArgument as e:
                    raise e.with_traceback(tb)

                raise

        api_rule_check_wrapper._rq_exception_checked = True
        return api_rule_check_wrapper

    return decorator



import numpy as np
import talib
import math


def get_extreme(array_high_price_result, array_low_price_result):
    np_array_high_price_result = np.array(array_high_price_result[:-1])
    np_array_low_price_result = np.array(array_low_price_result[:-1])
    max_result = np_array_high_price_result.max()
    min_result = np_array_low_price_result.min()
    return [max_result, min_result]


def get_atr_and_unit( atr_array_result,  atr_length_result, portfolio_value_result):
    atr = atr_array_result[ atr_length_result-1]
    unit = math.floor(portfolio_value_result * .01 / atr)
    return [atr, unit]


def get_stop_price(first_open_price_result, units_hold_result, atr_result):
    stop_price = first_open_price_result - 2 * atr_result \
                 + (units_hold_result - 1) * 0.5 * atr_result
    return stop_price


def init(context):
    context.trade_day_num = 0
    context.unit = 0
    context.atr = 0
    context.trading_signal = 'start'
    context.pre_trading_signal = ''
    context.units_hold_max = 4
    context.units_hold = 0
    context.quantity = 0
    context.max_add = 0
    context.first_open_price = 0
    context.s = '000300.XSHG'
    context.open_observe_time = 55
    context.close_observe_time = 20
    context.atr_time = 20


def handle_bar(context, bar_dict):
    portfolio_value = context.portfolio.portfolio_value
    high_price = history_bars(context.s, context.open_observe_time+1, '1d', 'high')
    low_price_for_atr = history_bars(context.s, context.open_observe_time+1, '1d', 'low')
    low_price_for_extreme = history_bars(context.s, context.close_observe_time+1, '1d', 'low')
    close_price = history_bars(context.s, context.open_observe_time+2, '1d', 'close')
    close_price_for_atr = close_price[:-1]

    atr_array = talib.ATR(high_price, low_price_for_atr, close_price_for_atr, timeperiod=context.atr_time)

    maxx = get_extreme(high_price, low_price_for_extreme)[0]
    minn = get_extreme(high_price, low_price_for_extreme)[1]
    atr = atr_array[-2]

    if context.trading_signal != 'start':
        if context.units_hold != 0:
            context.max_add += 0.5 * get_atr_and_unit(atr_array, atr_array.size, portfolio_value)[0]
    else:
        context.max_add = bar_dict[context.s].last

    cur_position = context.portfolio.positions[context.s].quantity
    available_cash = context.portfolio.cash
    market_value = context.portfolio.market_value

    if (cur_position > 0 and
            bar_dict[context.s].last < get_stop_price(context.first_open_price, context.units_hold, atr)):
        context.trading_signal = 'stop'
    else:
        if cur_position > 0 and bar_dict[context.s].last < minn:
            context.trading_signal = 'exit'
        else:
            if (bar_dict[context.s].last > context.max_add and context.units_hold != 0 and
                    context.units_hold < context.units_hold_max and
                    available_cash > bar_dict[context.s].last*context.unit):
                context.trading_signal = 'entry_add'
            else:
                if bar_dict[context.s].last > maxx and context.units_hold == 0:
                    context.max_add = bar_dict[context.s].last
                    context.trading_signal = 'entry'

    atr = get_atr_and_unit(atr_array, atr_array.size, portfolio_value)[0]
    if context.trade_day_num % 5 == 0:
        context.unit = get_atr_and_unit(atr_array, atr_array.size, portfolio_value)[1]
    context.trade_day_num += 1
    context.quantity = context.unit

    if (context.trading_signal != context.pre_trading_signal or
            (context.units_hold < context.units_hold_max and context.units_hold > 1) or
            context.trading_signal == 'stop'):
        if context.trading_signal == 'entry':
            context.quantity = context.unit
            if available_cash > bar_dict[context.s].last*context.quantity:
                order_shares(context.s, context.quantity)
                context.first_open_price = bar_dict[context.s].last
                context.units_hold = 1

        if context.trading_signal == 'entry_add':
            context.quantity = context.unit
            order_shares(context.s, context.quantity)
            context.units_hold += 1

        if context.trading_signal == 'stop':
            if context.units_hold > 0:
                order_shares(context.s, -context.quantity)
                context.units_hold -= 1

        if context.trading_signal == 'exit':
            if cur_position > 0:
                order_shares(context.s, -cur_position)
                context.units_hold = 0

    context.pre_trading_signal = context.trading_signal


__config__ = {
    "base": {
        "securities": "stock",
        "start_date": "2008-07-01",
        "end_date": "2014-09-01",
        "frequency": "1d",
        "matching_type": "current_bar",
        "stock_starting_cash": 1000000,
        "benchmark": "000300.XSHG",
    },
    "extra": {
        "log_level": "error",
    },
    "mod": {
        "sys_progress": {
            "enabled": True,
            "show": True,
        },
    },
}



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum


class CustomEnum(Enum):
    def __repr__(self):
        return "%s.%s" % (
            self.__class__.__name__, self._name_)


# noinspection PyPep8Naming
class EXECUTION_PHASE(CustomEnum):
    GLOBAL = "[全局]"
    ON_INIT = "[程序初始化]"
    BEFORE_TRADING = "[日内交易前]"
    ON_BAR = "[盘中 handle_bar 函数]"
    ON_TICK = "[盘中 handle_tick 函数]"
    AFTER_TRADING = "[日内交易后]"
    FINALIZED = "[程序结束]"
    SCHEDULED = "[scheduler函数内]"


# noinspection PyPep8Naming
class RUN_TYPE(CustomEnum):
    # TODO 取消 RUN_TYPE, 取而代之的是使用开启哪些Mod来控制策略所运行的类型
    # Back Test
    BACKTEST = "BACKTEST"
    # Paper Trading
    PAPER_TRADING = "PAPER_TRADING"
    LIVE_TRADING = 'LIVE_TRADING'


# noinspection PyPep8Naming
class ACCOUNT_TYPE(CustomEnum):
    TOTAL = 0
    BENCHMARK = 1
    STOCK = 2
    FUTURE = 3


# noinspection PyPep8Naming
class BAR_STATUS(CustomEnum):
    LIMIT_UP = "LIMIT_UP"
    LIMIT_DOWN = "LIMIT_DOWN"
    NORMAL = "NORMAL"
    ERROR = "ERROR"


# noinspection PyPep8Naming
class MATCHING_TYPE(CustomEnum):
    CURRENT_BAR_CLOSE = "CURRENT_BAR_CLOSE"
    NEXT_BAR_OPEN = "NEXT_BAR_OPEN"


# noinspection PyPep8Naming
class ORDER_TYPE(CustomEnum):
    MARKET = "MARKET"
    LIMIT = "LIMIT"


# noinspection PyPep8Naming
class ORDER_STATUS(CustomEnum):
    PENDING_NEW = "PENDING_NEW"
    ACTIVE = "ACTIVE"
    FILLED = "FILLED"
    REJECTED = "REJECTED"
    PENDING_CANCEL = "PENDING_CANCEL"
    CANCELLED = "CANCELLED"


# noinspection PyPep8Naming
class SIDE(CustomEnum):
    BUY = "BUY"
    SELL = "SELL"


# noinspection PyPep8Naming
class POSITION_EFFECT(CustomEnum):
    OPEN = "OPEN"
    CLOSE = "CLOSE"
    CLOSE_TODAY = "CLOSE_TODAY"


# noinspection PyPep8Naming
class EXC_TYPE(CustomEnum):
    USER_EXC = "USER_EXC"
    SYSTEM_EXC = "SYSTEM_EXC"
    NOTSET = "NOTSET"


# noinspection PyPep8Naming
class INSTRUMENT_TYPE(CustomEnum):
    CS = "CS"
    FUTURE = "FUTURE"
    OPTION = "OPTION"
    ETF = "ETF"
    LOF = "LOF"
    INDX = "INDX"
    FENJI_MU = "FENJI_MU"
    FENJI_A = "FENJI_A"
    FENJI_B = "FENJI_B"


# noinspection PyPep8Naming
class PERSIST_MODE(CustomEnum):
    ON_CRASH = "ON_CRASH"
    REAL_TIME = "REAL_TIME"


# noinspection PyPep8Naming
class MARGIN_TYPE(CustomEnum):
    BY_MONEY = "BY_MONEY"
    BY_VOLUME = "BY_VOLUME"


# noinspection PyPep8Naming
class COMMISSION_TYPE(CustomEnum):
    BY_MONEY = "BY_MONEY"
    BY_VOLUME = "BY_VOLUME"


# noinspection PyPep8Naming
class EXIT_CODE(CustomEnum):
    EXIT_SUCCESS = "EXIT_SUCCESS"
    EXIT_USER_ERROR = "EXIT_USER_ERROR"
    EXIT_INTERNAL_ERROR = "EXIT_INTERNAL_ERROR"


# noinspection PyPep8Naming
class HEDGE_TYPE(CustomEnum):
    HEDGE = "hedge"
    SPECULATION = "speculation"
    ARBITRAGE = "arbitrage"


# noinspection PyPep8Naming
class DAYS_CNT(object):
    DAYS_A_YEAR = 365
    TRADING_DAYS_A_YEAR = 252


UNDERLYING_SYMBOL_PATTERN = "([a-zA-Z]+)\d+"

NIGHT_TRADING_NS = ["CU", "AL", "ZN", "PB", "SN", "NI", "RB", "HC", "BU", "RU", "AU", "AG", "Y", "M", "A", "B", "P",
                    "J", "JM", "I", "CF", "SR", "OI", "MA", "ZC", "FG", "RM"]



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from rqalpha.const import ACCOUNT_TYPE

from .commission import StockCommission, FutureCommission
from .slippage import PriceRatioSlippage
from .tax import StockTax, FutureTax


class CommissionDecider(object):
    def __init__(self, multiplier):
        self.deciders = dict()
        self.deciders[ACCOUNT_TYPE.STOCK] = StockCommission(multiplier)
        self.deciders[ACCOUNT_TYPE.FUTURE] = FutureCommission(multiplier)

    def get_commission(self, account_type, trade):
        return self.deciders[account_type].get_commission(trade)


class SlippageDecider(object):
    def __init__(self, rate):
        self.decider = PriceRatioSlippage(rate)

    def get_trade_price(self, side, price):
        return self.decider.get_trade_price(side, price)


class TaxDecider(object):
    def __init__(self, rate=None):
        self.deciders = dict()
        self.deciders[ACCOUNT_TYPE.STOCK] = StockTax(rate)
        self.deciders[ACCOUNT_TYPE.BENCHMARK] = StockTax(rate)
        self.deciders[ACCOUNT_TYPE.FUTURE] = FutureTax(rate)

    def get_tax(self, account_type, trade):
        return self.deciders[account_type].get_tax(trade)



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import simplejson as json
import datetime
from .. import const


def convert_dict_to_json(dict_obj):
    dict_obj = json.dumps(dict_obj, default=custom_encode)
    return dict_obj


def convert_json_to_dict(json_str):
    dict_obj = json.loads(json_str, object_hook=custom_decode)
    return dict_obj


def custom_encode(obj):
    if isinstance(obj, datetime.datetime):
        obj = {"__datetime__": True, "as_str": obj.strftime("%Y%m%dT%H:%M:%S.%f")}
    elif isinstance(obj, datetime.date):
        obj = {"__date__": True, "as_str": obj.strftime("%Y%m%d")}
    elif isinstance(obj, const.CustomEnum):
        obj = {"__enum__": True, "as_str": str(obj)}
    else:
        raise TypeError(
            "Unserializable object {} of type {}".format(obj, type(obj))
        )
    return obj


def custom_decode(obj):
    if "__datetime__" in obj:
        obj = datetime.datetime.strptime(obj["as_str"], "%Y%m%dT%H:%M:%S.%f")
    elif "__date__" in obj:
        obj = datetime.datetime.strptime(obj["as_str"], "%Y%m%d").date()
    elif "__enum__" in obj:
        [e, v] = obj["as_str"].split(".")
        obj = getattr(getattr(const, e), v)
    return obj



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pickle

from ..model.instrument import Instrument


class InstrumentStore(object):
    def __init__(self, f):
        with open(f, 'rb') as store:
            d = pickle.load(store)
        self._instruments = [Instrument(i) for i in d]

    def get_all_instruments(self):
        return self._instruments



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six


class BaseAccount(object):
    def __init__(self, total_cash, positions, backward_trade_set=set(), register_event=True):
        self._positions = positions
        self._frozen_cash = 0
        self._total_cash = total_cash
        self._backward_trade_set = backward_trade_set
        if register_event:
            self.register_event()

    def register_event(self):
        """
        注册事件
        """
        raise NotImplementedError

    def fast_forward(self, orders, trades=list()):
        """
        同步账户信息至最新状态
        :param orders: 订单列表，主要用来计算frozen_cash，如果为None则不计算frozen_cash
        :param trades: 交易列表，基于Trades 将当前Positions ==> 最新Positions
        """
        raise NotImplementedError

    @property
    def type(self):
        """
        [enum] 账户类型
        """
        raise NotImplementedError

    @property
    def total_value(self):
        """
        [float]总权益
        """
        raise NotImplementedError

    def get_state(self):
        raise NotImplementedError

    def set_state(self, state):
        raise NotImplementedError

    @property
    def positions(self):
        """
        [dict] 持仓
        """
        return self._positions

    @property
    def frozen_cash(self):
        """
        [float] 冻结资金
        """
        return self._frozen_cash

    @property
    def cash(self):
        """
        [float] 可用资金
        """
        return self._total_cash - self._frozen_cash

    @property
    def market_value(self):
        """
        [float] 市值
        """
        return sum(position.market_value for position in six.itervalues(self._positions))

    @property
    def transaction_cost(self):
        """
        [float] 总费用
        """
        return sum(position.transaction_cost for position in six.itervalues(self._positions))




# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from ..const import COMMISSION_TYPE, MARGIN_TYPE


CN_FUTURE_INFO = {
    "SM": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 3.0
        }
    },
    "SR": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 3.0
        }
    },
    "JD": {
        "hedge": {
            "short_margin_ratio": 0.08,
            "close_commission_today_ratio": 0.00015,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 0.00015,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.08,
            "open_commission_ratio": 0.00015
        },
        "speculation": {
            "short_margin_ratio": 0.08,
            "close_commission_today_ratio": 0.00015,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 0.00015,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.08,
            "open_commission_ratio": 0.00015
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.00015,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 0.00015,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 0.00015
        }
    },
    "T": {
        "hedge": {
            "short_margin_ratio": 0.02,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.02,
            "open_commission_ratio": 3.0
        },
        "speculation": {
            "short_margin_ratio": 0.02,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.02,
            "open_commission_ratio": 3.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.02,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.02,
            "open_commission_ratio": 3.0
        }
    },
    "P": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.5
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.5
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 2.5
        }
    },
    "BB": {
        "hedge": {
            "short_margin_ratio": 0.2,
            "close_commission_today_ratio": 5e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 0.0001,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.2,
            "open_commission_ratio": 0.0001
        },
        "speculation": {
            "short_margin_ratio": 0.2,
            "close_commission_today_ratio": 5e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 0.0001,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.2,
            "open_commission_ratio": 0.0001
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 5e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 0.0001,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 0.0001
        }
    },
    "RM": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 1.5
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 1.5
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.5,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 1.5
        }
    },
    "RS": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 2.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 2.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 2.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 2.0
        }
    },
    "J": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 3e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 6e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 6e-05
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 3e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 6e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 6e-05
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 3e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 6e-05,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 6e-05
        }
    },
    "RI": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 2.5,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.5
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 2.5,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.5
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 2.5,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 2.5
        }
    },
    "ER": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 2.5,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.5
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 2.5,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.5
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 2.5,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 2.5
        }
    },
    "L": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 2.0
        }
    },
    "PP": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 2.5e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 5e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 5e-05
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 2.5e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 5e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 5e-05
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 2.5e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 5e-05,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 5e-05
        }
    },
    "SN": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 3.0
        }
    },
    "I": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 3e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 6e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 6e-05
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 3e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 6e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 6e-05
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 3e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 6e-05,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 6e-05
        }
    },
    "TA": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 3.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 3.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 3.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 3.0
        }
    },
    "AL": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 3.0
        }
    },
    "ZC": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 4.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 4.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 4.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 4.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 4.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 4.0
        }
    },
    "TC": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 4.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 4.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 4.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 4.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 4.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 4.0
        }
    },
    "LR": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 3.0
        }
    },
    "RU": {
        "hedge": {
            "short_margin_ratio": 0.08,
            "close_commission_today_ratio": 4.5e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 4.5e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.08,
            "open_commission_ratio": 4.5e-05
        },
        "speculation": {
            "short_margin_ratio": 0.08,
            "close_commission_today_ratio": 4.5e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 4.5e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.08,
            "open_commission_ratio": 4.5e-05
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 4.5e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 4.5e-05,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 4.5e-05
        }
    },
    "M": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 1.5
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 1.5
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.5,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 1.5
        }
    },
    "TF": {
        "hedge": {
            "short_margin_ratio": 0.012,
            "close_commission_today_ratio": 3.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.012,
            "open_commission_ratio": 3.0
        },
        "speculation": {
            "short_margin_ratio": 0.012,
            "close_commission_today_ratio": 3.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.012,
            "open_commission_ratio": 3.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.012,
            "close_commission_today_ratio": 3.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.012,
            "open_commission_ratio": 3.0
        }
    },
    "MA": {
        "hedge": {
            "short_margin_ratio": 0.07,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.4,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.07,
            "open_commission_ratio": 1.4
        },
        "speculation": {
            "short_margin_ratio": 0.07,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.4,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.07,
            "open_commission_ratio": 1.4
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.4,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 1.4
        }
    },
    "ME": {
        "hedge": {
            "short_margin_ratio": 0.07,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.4,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.07,
            "open_commission_ratio": 1.4
        },
        "speculation": {
            "short_margin_ratio": 0.07,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.4,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.07,
            "open_commission_ratio": 1.4
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.4,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 1.4
        }
    },
    "WR": {
        "hedge": {
            "short_margin_ratio": 0.2,
            "close_commission_today_ratio": 4e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 4e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.2,
            "open_commission_ratio": 4e-05
        },
        "speculation": {
            "short_margin_ratio": 0.2,
            "close_commission_today_ratio": 4e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 4e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.2,
            "open_commission_ratio": 4e-05
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 4e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 4e-05,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 4e-05
        }
    },
    "RB": {
        "hedge": {
            "short_margin_ratio": 0.06,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 4.5e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.06,
            "open_commission_ratio": 4.5e-05
        },
        "speculation": {
            "short_margin_ratio": 0.06,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 4.5e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.06,
            "open_commission_ratio": 4.5e-05
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 4.5e-05,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 4.5e-05
        }
    },
    "C": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.2,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 1.2
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.2,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 1.2
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.2,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 1.2
        }
    },
    "JR": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 3.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 3.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 3.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 3.0
        }
    },
    "SF": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 3.0
        }
    },
    "OI": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.5
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.5
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 2.5
        }
    },
    "RO": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.5
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.5
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 2.5
        }
    },
    "CF": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 4.3,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 4.3
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 4.3,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 4.3
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 4.3,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 4.3
        }
    },
    "BU": {
        "hedge": {
            "short_margin_ratio": 0.07,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 3e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.07,
            "open_commission_ratio": 3e-05
        },
        "speculation": {
            "short_margin_ratio": 0.07,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 3e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.07,
            "open_commission_ratio": 3e-05
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 3e-05,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 3e-05
        }
    },
    "JM": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 3e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 6e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 6e-05
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 3e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 6e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 6e-05
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 3e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 6e-05,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 6e-05
        }
    },
    "IH": {
        "hedge": {
            "short_margin_ratio": 0.2,
            "close_commission_today_ratio": 0.000115,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 2.5e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.2,
            "open_commission_ratio": 2.5e-05
        },
        "speculation": {
            "short_margin_ratio": 0.4,
            "close_commission_today_ratio": 0.0023,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 2.3e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.4,
            "open_commission_ratio": 2.3e-05
        },
        "arbitrage": {
            "short_margin_ratio": 0.2,
            "close_commission_today_ratio": 0.000115,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 2.5e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.2,
            "open_commission_ratio": 2.5e-05
        }
    },
    "FG": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 3.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 3.0
        }
    },
    "PM": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 5.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 5.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 5.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 5.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 5.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 5.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 5.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 5.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 5.0
        }
    },
    "FB": {
        "hedge": {
            "short_margin_ratio": 0.2,
            "close_commission_today_ratio": 5e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 0.0001,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.2,
            "open_commission_ratio": 0.0001
        },
        "speculation": {
            "short_margin_ratio": 0.2,
            "close_commission_today_ratio": 5e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 0.0001,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.2,
            "open_commission_ratio": 0.0001
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 5e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 0.0001,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 0.0001
        }
    },
    "CS": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 1.5
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 1.5
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 1.5,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 1.5
        }
    },
    "B": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 2.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 2.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 2.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 2.0
        }
    },
    "IC": {
        "hedge": {
            "short_margin_ratio": 0.2,
            "close_commission_today_ratio": 0.000115,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 2.5e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.2,
            "open_commission_ratio": 2.5e-05
        },
        "speculation": {
            "short_margin_ratio": 0.4,
            "close_commission_today_ratio": 0.0023,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 2.3e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.4,
            "open_commission_ratio": 2.3e-05
        },
        "arbitrage": {
            "short_margin_ratio": 0.2,
            "close_commission_today_ratio": 0.000115,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 2.5e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.2,
            "open_commission_ratio": 2.5e-05
        }
    },
    "WH": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.5
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.5
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 2.5
        }
    },
    "WS": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.5
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.5
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 2.5
        }
    },
    "FU": {
        "hedge": {
            "short_margin_ratio": 0.2,
            "close_commission_today_ratio": 2e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 2e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.2,
            "open_commission_ratio": 2e-05
        },
        "speculation": {
            "short_margin_ratio": 0.2,
            "close_commission_today_ratio": 2e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 2e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.2,
            "open_commission_ratio": 2e-05
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 2e-05,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 2e-05,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 2e-05
        }
    },
    "AU": {
        "hedge": {
            "short_margin_ratio": 0.06,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 10.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.06,
            "open_commission_ratio": 10.0
        },
        "speculation": {
            "short_margin_ratio": 0.06,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 10.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.06,
            "open_commission_ratio": 10.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 10.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 10.0
        }
    },
    "CU": {
        "hedge": {
            "short_margin_ratio": 0.08,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 2.5e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.08,
            "open_commission_ratio": 2.5e-05
        },
        "speculation": {
            "short_margin_ratio": 0.08,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 2.5e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.08,
            "open_commission_ratio": 2.5e-05
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 2.5e-05,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 2.5e-05
        }
    },
    "V": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 2.0
        }
    },
    "Y": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.5
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.5
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.5,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 2.5
        }
    },
    "AG": {
        "hedge": {
            "short_margin_ratio": 0.08,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 5e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.08,
            "open_commission_ratio": 5e-05
        },
        "speculation": {
            "short_margin_ratio": 0.08,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 5e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.08,
            "open_commission_ratio": 5e-05
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 5e-05,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 5e-05
        }
    },
    "PB": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 4e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 4e-05
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 4e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 4e-05
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 4e-05,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 4e-05
        }
    },
    "IF": {
        "hedge": {
            "short_margin_ratio": 0.2,
            "close_commission_today_ratio": 0.000115,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 2.5e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.2,
            "open_commission_ratio": 2.5e-05
        },
        "speculation": {
            "short_margin_ratio": 0.4,
            "close_commission_today_ratio": 0.0023,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 2.3e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.4,
            "open_commission_ratio": 2.3e-05
        },
        "arbitrage": {
            "short_margin_ratio": 0.2,
            "close_commission_today_ratio": 0.000115,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 2.5e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.2,
            "open_commission_ratio": 2.5e-05
        }
    },
    "A": {
        "hedge": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.0
        },
        "speculation": {
            "short_margin_ratio": 0.05,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.05,
            "open_commission_ratio": 2.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 2.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 2.0
        }
    },
    "NI": {
        "hedge": {
            "short_margin_ratio": 0.07,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 6.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.07,
            "open_commission_ratio": 6.0
        },
        "speculation": {
            "short_margin_ratio": 0.07,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 6.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.07,
            "open_commission_ratio": 6.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 6.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 6.0
        }
    },
    "HC": {
        "hedge": {
            "short_margin_ratio": 0.06,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 4e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.06,
            "open_commission_ratio": 4e-05
        },
        "speculation": {
            "short_margin_ratio": 0.06,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 4e-05,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.06,
            "open_commission_ratio": 4e-05
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_MONEY,
            "close_commission_ratio": 4e-05,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 4e-05
        }
    },
    "ZN": {
        "hedge": {
            "short_margin_ratio": 0.06,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.06,
            "open_commission_ratio": 3.0
        },
        "speculation": {
            "short_margin_ratio": 0.06,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": MARGIN_TYPE.BY_MONEY,
            "long_margin_ratio": 0.06,
            "open_commission_ratio": 3.0
        },
        "arbitrage": {
            "short_margin_ratio": 0.0,
            "close_commission_today_ratio": 0.0,
            "commission_type": COMMISSION_TYPE.BY_VOLUME,
            "close_commission_ratio": 3.0,
            "margin_type": None,
            "long_margin_ratio": 0.0,
            "open_commission_ratio": 3.0
        }
    }
}




# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six
import pickle
from ..utils.logger import user_system_log, system_log, user_detail_log


class GlobalVars(object):
    def get_state(self):
        dict_data = {}
        for key, value in six.iteritems(self.__dict__):
            try:
                dict_data[key] = pickle.dumps(value)
            except Exception as e:
                user_detail_log.exception("g.{} can not pickle", key)
                user_system_log.warn("g.{} can not pickle", key)
        return pickle.dumps(dict_data)

    def set_state(self, state):
        dict_data = pickle.loads(state)
        for key, value in six.iteritems(dict_data):
            try:
                self.__dict__[key] = pickle.loads(value)
                system_log.debug("restore g.{} {}", key, type(self.__dict__[key]))
            except Exception as e:
                user_detail_log.exception("g.{} can not restore", key)
                user_system_log.warn("g.{} can not restore", key)



#!/usr/bin/env python
# encoding: utf-8
import inspect

def test_buy_open():
    from rqalpha.api import buy_open, subscribe, get_order, ORDER_STATUS, POSITION_EFFECT, SIDE
    def init(context):
        context.f1 = 'P88'
        context.amount = 1
        # context.marin_rate = 10
        subscribe(context.f1)
        context.order_count = 0
        context.order = None

    def handle_bar(context, bar_dict):
        order_id = buy_open(context.f1, 1)
        order = get_order(order_id)
        assert order.order_book_id == context.f1
        assert order.quantity == 1
        assert order.status == ORDER_STATUS.ACTIVE
        assert order.unfilled_quantity == 1
        assert order.unfilled_quantity + order.filled_quantity == order.quantity
        assert order.side == SIDE.BUY
        assert order.position_effect == POSITION_EFFECT.OPEN
test_buy_open_code_new = "".join(inspect.getsourcelines(test_buy_open)[0])


def test_sell_open():
    from rqalpha.api import sell_open, subscribe, get_order, ORDER_STATUS, POSITION_EFFECT, SIDE
    def init(context):
        context.f1 = 'P88'
        context.amount = 1
        # context.marin_rate = 10
        subscribe(context.f1)
        context.order_count = 0
        context.order = None

    def handle_bar(context, bar_dict):
        order_id = sell_open(context.f1, 1)
        order = get_order(order_id)
        assert order.order_book_id == context.f1
        assert order.quantity == 1
        assert order.status == ORDER_STATUS.ACTIVE
        assert order.unfilled_quantity == 1
        assert order.unfilled_quantity + order.filled_quantity == order.quantity
        assert order.side == SIDE.SELL
        assert order.position_effect == POSITION_EFFECT.OPEN
test_sell_open_code_new = "".join(inspect.getsourcelines(test_sell_open)[0])


def test_buy_close():
    from rqalpha.api import buy_close, subscribe, get_order, ORDER_STATUS, POSITION_EFFECT, SIDE
    def init(context):
        context.f1 = 'P88'
        context.amount = 1
        # context.marin_rate = 10
        subscribe(context.f1)
        context.order_count = 0
        context.order = None

    def handle_bar(context, bar_dict):
        order_id = buy_close(context.f1, 1)
        order = get_order(order_id)
        assert order.order_book_id == context.f1
        assert order.quantity == 1
        assert order.status == ORDER_STATUS.ACTIVE
        assert order.unfilled_quantity == 1
        assert order.unfilled_quantity + order.filled_quantity == order.quantity
        assert order.side == SIDE.BUY
        assert order.position_effect == POSITION_EFFECT.CLOSE
test_buy_close_code_new = "".join(inspect.getsourcelines(test_buy_close)[0])


def test_sell_close():
    from rqalpha.api import sell_close, subscribe, get_order, ORDER_STATUS, POSITION_EFFECT, SIDE
    def init(context):
        context.f1 = 'P88'
        context.amount = 1
        # context.marin_rate = 10
        subscribe(context.f1)
        context.order_count = 0
        context.order = None

    def handle_bar(context, bar_dict):
        order_id = sell_close(context.f1, 1)
        order = get_order(order_id)
        assert order.order_book_id == context.f1
        assert order.quantity == 1
        assert order.status == ORDER_STATUS.ACTIVE
        assert order.unfilled_quantity == 1
        assert order.unfilled_quantity + order.filled_quantity == order.quantity
        assert order.side == SIDE.SELL
        assert order.position_effect == POSITION_EFFECT.CLOSE
test_sell_close_code_new = "".join(inspect.getsourcelines(test_sell_close)[0])



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from rqalpha.interface import AbstractFrontendValidator
from rqalpha.const import SIDE, POSITION_EFFECT, ACCOUNT_TYPE

from rqalpha.utils.i18n import gettext as _


class PositionValidator(AbstractFrontendValidator):
    @staticmethod
    def _stock_validator(account, order):
        if order.side != SIDE.SELL:
            return True

        position = account.positions[order.order_book_id]
        if order.quantity <= position.sellable:
            return True

        order.mark_rejected(_(
            "Order Rejected: not enough stock {order_book_id} to sell, you want to sell {quantity},"
            " sellable {sellable}").format(
            order_book_id=order.order_book_id,
            quantity=order.quantity,
            sellable=position.sellable,
        ))
        return False

    @staticmethod
    def _future_validator(account, order):
        if order.position_effect != POSITION_EFFECT.CLOSE:
            return True

        position = account.positions[order.order_book_id]
        if order.side == SIDE.BUY and order.quantity > position.closable_sell_quantity:
            order.mark_rejected(_(
                "Order Rejected: not enough securities {order_book_id} to buy close, target"
                " sell quantity is {quantity}, sell_closable_quantity {closable}").format(
                order_book_id=order.order_book_id,
                quantity=order.quantity,
                closable=position.closable_sell_quantity,
            ))
            return False
        elif order.side == SIDE.SELL and order.quantity > position.closable_buy_quantity:
            order.mark_rejected(_(
                "Order Rejected: not enough securities {order_book_id} to sell close, target"
                " sell quantity is {quantity}, buy_closable_quantity {closable}").format(
                order_book_id=order.order_book_id,
                quantity=order.quantity,
                closable=position.closable_buy_quantity,
            ))
            return False
        return True

    def can_submit_order(self, account, order):
        if account.type == ACCOUNT_TYPE.STOCK:
            return self._stock_validator(account, order)
        elif account.type == ACCOUNT_TYPE.FUTURE:
            return self._future_validator(account, order)
        else:
            raise NotImplementedError

    def can_cancel_order(self, account, order):
        return True



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import pickle
from collections import defaultdict
from enum import Enum

import numpy as np
import pandas as pd
import six

from rqalpha.const import EXIT_CODE, ACCOUNT_TYPE
from rqalpha.environment import Environment
from rqalpha.events import EVENT
from rqalpha.interface import AbstractMod
from rqalpha.utils.risk import Risk


class AnalyserMod(AbstractMod):
    def __init__(self):
        self._env = None
        self._mod_config = None
        self._enabled = False

        self._orders = []
        self._trades = []
        self._total_portfolios = []
        self._total_benchmark_portfolios = []
        self._sub_accounts = defaultdict(list)
        self._positions = defaultdict(list)

        self._benchmark_daily_returns = []
        self._portfolio_daily_returns = []

    def start_up(self, env, mod_config):
        self._env = env
        self._mod_config = mod_config
        self._enabled = (self._mod_config.record or self._mod_config.plot or self._mod_config.output_file or
                         self._mod_config.plot_save_file or self._mod_config.report_save_path)

        if self._enabled:
            env.event_bus.add_listener(EVENT.POST_SETTLEMENT, self._collect_daily)
            env.event_bus.add_listener(EVENT.TRADE, self._collect_trade)
            env.event_bus.add_listener(EVENT.ORDER_CREATION_PASS, self._collect_order)

    def _collect_trade(self, event):
        self._trades.append(self._to_trade_record(event.trade))

    def _collect_order(self, event):
        self._orders.append(event.order)

    def _collect_daily(self, event):
        date = self._env.calendar_dt.date()
        portfolio = self._env.portfolio
        benchmark_portfolio = self._env.benchmark_portfolio

        self._portfolio_daily_returns.append(portfolio.daily_returns)
        self._total_portfolios.append(self._to_portfolio_record(date, portfolio))

        if benchmark_portfolio is None:
            self._benchmark_daily_returns.append(0)
        else:
            self._benchmark_daily_returns.append(benchmark_portfolio.daily_returns)
            self._total_benchmark_portfolios.append(self._to_portfolio_record(date, benchmark_portfolio))

        for account_type, account in six.iteritems(self._env.portfolio.accounts):
            self._sub_accounts[account_type].append(self._to_account_record(date, account))
            for order_book_id, position in six.iteritems(account.positions):
                self._positions[account_type].append(self._to_position_record(date, order_book_id, position))

    def _symbol(self, order_book_id):
        return self._env.data_proxy.instruments(order_book_id).symbol

    @staticmethod
    def _safe_convert(value, ndigits=3):
        if isinstance(value, Enum):
            return value.name

        if isinstance(value, (float, np.float64, np.float32, np.float16, np.float)):
            return round(value, ndigits)

        return value

    def _to_portfolio_record(self, date, portfolio):
        return {
            'date': date,
            'cash': self._safe_convert(portfolio.cash),
            'total_returns': self._safe_convert(portfolio.total_returns),
            'daily_returns': self._safe_convert(portfolio.daily_returns),
            'daily_pnl': self._safe_convert(portfolio.daily_pnl),
            'total_value': self._safe_convert(portfolio.total_value),
            'market_value': self._safe_convert(portfolio.market_value),
            'annualized_returns': self._safe_convert(portfolio.annualized_returns),
            'unit_net_value': self._safe_convert(portfolio.unit_net_value),
            'units': portfolio.units,
            'static_unit_net_value': self._safe_convert(portfolio.static_unit_net_value),
        }

    ACCOUNT_FIELDS_MAP = {
        ACCOUNT_TYPE.STOCK: ['dividend_receivable'],
        ACCOUNT_TYPE.FUTURE: ['holding_pnl', 'realized_pnl', 'daily_pnl', 'margin'],
    }

    def _to_account_record(self, date, account):
        data = {
            'date': date,
            'total_cash': self._safe_convert(account.cash + account.frozen_cash),
            'transaction_cost': self._safe_convert(account.transaction_cost),
            'market_value': self._safe_convert(account.market_value),
            'total_value': self._safe_convert(account.total_value),
        }

        for f in self.ACCOUNT_FIELDS_MAP[account.type]:
            data[f] = self._safe_convert(getattr(account, f))

        return data

    POSITION_FIELDS_MAP = {
        ACCOUNT_TYPE.STOCK: [
            'quantity', 'last_price', 'avg_price', 'market_value', 'sellable'
        ],
        ACCOUNT_TYPE.FUTURE: [
            'pnl', 'daily_pnl', 'holding_pnl', 'realized_pnl', 'margin', 'market_value',
            'buy_pnl', 'sell_pnl', 'closable_buy_quantity', 'buy_margin', 'buy_today_quantity',
            'buy_avg_open_price', 'buy_avg_holding_price', 'closable_sell_quantity',
            'sell_margin', 'sell_today_quantity', 'sell_quantity', 'sell_avg_open_price',
            'sell_avg_holding_price'
        ],
    }

    def _to_position_record(self, date, order_book_id, position):
        data = {
            'order_book_id': order_book_id,
            'symbol': self._symbol(order_book_id),
            'date': date,
        }

        for f in self.POSITION_FIELDS_MAP[position.type]:
            data[f] = self._safe_convert(getattr(position, f))
        return data

    def _to_trade_record(self, trade):
        return {
            'datetime': trade.datetime.strftime("%Y-%m-%d %H:%M:%S"),
            'trading_datetime': trade.trading_datetime.strftime("%Y-%m-%d %H:%M:%S"),
            'order_book_id': trade.order_book_id,
            'symbol': self._symbol(trade.order_book_id),
            'side': self._safe_convert(trade.side),
            'position_effect': self._safe_convert(trade.position_effect),
            'exec_id': trade.exec_id,
            'tax': trade.tax,
            'commission': trade.commission,
            'last_quantity': trade.last_quantity,
            'last_price': self._safe_convert(trade.last_price),
            'order_id': trade.order_id,
            'transaction_cost': trade.transaction_cost,
        }

    def tear_down(self, code, exception=None):
        if code != EXIT_CODE.EXIT_SUCCESS or not self._enabled:
            return

        strategy_name = os.path.basename(self._env.config.base.strategy_file).split(".")[0]
        data_proxy = self._env.data_proxy

        summary = {
            'strategy_name': strategy_name,
            'start_date': self._env.config.base.start_date.strftime('%Y-%m-%d'),
            'end_date': self._env.config.base.end_date.strftime('%Y-%m-%d'),
            'strategy_file': self._env.config.base.strategy_file,
            'securities': self._env.config.base.securities,
            'run_type': self._env.config.base.run_type.value,
            'stock_starting_cash': self._env.config.base.stock_starting_cash,
            'future_starting_cash': self._env.config.base.future_starting_cash,
        }

        risk = Risk(np.array(self._portfolio_daily_returns), np.array(self._benchmark_daily_returns),
                    data_proxy.get_risk_free_rate(self._env.config.base.start_date, self._env.config.base.end_date),
                    (self._env.config.base.end_date - self._env.config.base.start_date).days + 1)
        summary.update({
            'alpha': self._safe_convert(risk.alpha, 3),
            'beta': self._safe_convert(risk.beta, 3),
            'sharpe': self._safe_convert(risk.sharpe, 3),
            'information_ratio': self._safe_convert(risk.information_ratio, 3),
            'downside_risk': self._safe_convert(risk.annual_downside_risk, 3),
            'tracking_error': self._safe_convert(risk.annual_tracking_error, 3),
            'sortino': self._safe_convert(risk.sortino, 3),
            'volatility': self._safe_convert(risk.annual_volatility, 3),
            'max_drawdown': self._safe_convert(risk.max_drawdown, 3),
        })

        summary.update({
            'total_value': self._safe_convert(self._env.portfolio.total_value),
            'cash': self._safe_convert(self._env.portfolio.cash),
            'total_returns': self._safe_convert(self._env.portfolio.total_returns),
            'annualized_returns': self._safe_convert(self._env.portfolio.annualized_returns),
            'unit_net_value': self._safe_convert(self._env.portfolio.unit_net_value),
            'units': self._env.portfolio.units,
        })

        if self._env.benchmark_portfolio:
            summary['benchmark_total_returns'] = self._safe_convert(self._env.benchmark_portfolio.total_returns)
            summary['benchmark_annualized_returns'] = self._safe_convert(
                self._env.benchmark_portfolio.annualized_returns)

        trades = pd.DataFrame(self._trades)
        if 'datetime' in trades.columns:
            trades = trades.set_index('datetime')

        df = pd.DataFrame(self._total_portfolios)
        df['date'] = pd.to_datetime(df['date'])
        total_portfolios = df.set_index('date').sort_index()

        result_dict = {
            'summary': summary,
            'trades': trades,
            'total_portfolios': total_portfolios,
        }

        if self._env.benchmark_portfolio is not None:
            b_df = pd.DataFrame(self._total_benchmark_portfolios)
            df['date'] = pd.to_datetime(df['date'])
            benchmark_portfolios = b_df.set_index('date').sort_index()
            result_dict['benchmark_portfolios'] = benchmark_portfolios

        if Environment.get_instance().plot_store is not None:
            plots = Environment.get_instance().get_plot_store().get_plots()
            plots_items = defaultdict(dict)
            for series_name, value_dict in six.iteritems(plots):
                for date, value in six.iteritems(value_dict):
                    plots_items[date][series_name] = value
                    plots_items[date]["date"] = date

            df = pd.DataFrame([dict_data for date, dict_data in six.iteritems(plots_items)])
            df["date"] = pd.to_datetime(df["date"])
            df = df.set_index("date").sort_index()
            result_dict["plots"] = df

        for account_type, account in six.iteritems(self._env.portfolio.accounts):
            account_name = account_type.name.lower()
            portfolios_list = self._sub_accounts[account_type]
            df = pd.DataFrame(portfolios_list)
            df["date"] = pd.to_datetime(df["date"])
            portfolios_df = df.set_index("date").sort_index()
            result_dict["{}_portfolios".format(account_name)] = portfolios_df

            positions_list = self._positions[account_type]
            positions_df = pd.DataFrame(positions_list)
            if "date" in positions_df.columns:
                positions_df["date"] = pd.to_datetime(positions_df["date"])
                positions_df = positions_df.set_index("date").sort_index()
            result_dict["{}_positions".format(account_name)] = positions_df

        if self._mod_config.output_file:
            with open(self._mod_config.output_file, 'wb') as f:
                pickle.dump(result_dict, f)

        if self._mod_config.report_save_path:
            from .report import generate_report
            generate_report(result_dict, self._mod_config.report_save_path)

        if self._mod_config.plot or self._mod_config.plot_save_file:
            from .plot import plot_result
            plot_result(result_dict, self._mod_config.plot, self._mod_config.plot_save_file)

        return result_dict



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import math
import time
import datetime
from six.moves import reduce

from rqalpha.utils.datetime_func import convert_dt_to_int


def is_holiday_today():
    today = datetime.date.today()
    from rqalpha.environment import Environment
    return not Environment.get_instance().data_proxy.is_trading_date(today)


def is_tradetime_now():
    now_time = time.localtime()
    now = (now_time.tm_hour, now_time.tm_min, now_time.tm_sec)
    if (9, 15, 0) <= now <= (11, 30, 0) or (13, 0, 0) <= now <= (15, 0, 0):
        return True
    return False


TUSHARE_CODE_MAPPING = {
    "sh": "000001.XSHG",
    "sz": "399001.XSHE",
    "sz50": "000016.XSHG",
    "hs300": "000300.XSHG",
    "sz500": "000905.XSHG",
    "zxb": "399005.XSHE",
    "cyb": "399006.XSHE",
}


def tushare_code_2_order_book_id(code):
    try:
        return TUSHARE_CODE_MAPPING[code]
    except KeyError:
        if code.startswith("6"):
            return "{}.XSHG".format(code)
        elif code[0] in ["3", "0"]:
            return "{}.XSHE".format(code)
        else:
            raise RuntimeError("Unknown code")


def order_book_id_2_tushare_code(order_book_id):
    return order_book_id.split(".")[0]


def get_realtime_quotes(code_list, open_only=False):
    import tushare as ts

    max_len = 800
    loop_cnt = int(math.ceil(float(len(code_list)) / max_len))

    total_df = reduce(lambda df1, df2: df1.append(df2),
                      [ts.get_realtime_quotes([code for code in code_list[i::loop_cnt]])
                       for i in range(loop_cnt)])
    total_df["is_index"] = False

    index_symbol = ["sh", "sz", "hs300", "sz50", "zxb", "cyb"]
    index_df = ts.get_realtime_quotes(index_symbol)
    index_df["code"] = index_symbol
    index_df["is_index"] = True
    total_df = total_df.append(index_df)
    total_df = total_df.set_index("code").sort_index()

    columns = set(total_df.columns) - set(["name", "time", "date"])
    # columns = filter(lambda x: "_v" not in x, columns)
    for label in columns:
        total_df[label] = total_df[label].map(lambda x: 0 if str(x).strip() == "" else x)
        total_df[label] = total_df[label].astype(float)

    total_df["chg"] = total_df["price"] / total_df["pre_close"] - 1

    total_df["order_book_id"] = total_df.index
    total_df["order_book_id"] = total_df["order_book_id"].apply(tushare_code_2_order_book_id)

    total_df["datetime"] = total_df["date"] + " " + total_df["time"]
    total_df["datetime"] = total_df["datetime"].apply(lambda x: convert_dt_to_int(datetime.datetime.strptime(x, "%Y-%m-%d %H:%M:%S")))

    total_df["close"] = total_df["price"]

    if open_only:
        total_df = total_df[total_df.open > 0]

    return total_df



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from rqalpha.const import ACCOUNT_TYPE
from rqalpha.model.account import StockAccount, FutureAccount
from rqalpha.model.position import Positions, StockPosition, FuturePosition
from rqalpha.model.portfolio import Portfolio


def init_portfolio(env):
    accounts = {}
    config = env.config
    start_date = config.base.start_date
    total_cash = 0
    for account_type in config.base.account_list:
        if account_type == ACCOUNT_TYPE.STOCK:
            stock_starting_cash = config.base.stock_starting_cash
            accounts[ACCOUNT_TYPE.STOCK] = StockAccount(stock_starting_cash, Positions(StockPosition))
            total_cash += stock_starting_cash
        elif account_type == ACCOUNT_TYPE.FUTURE:
            future_starting_cash = config.base.future_starting_cash
            accounts[ACCOUNT_TYPE.FUTURE] = FutureAccount(future_starting_cash, Positions(FuturePosition))
            total_cash += future_starting_cash
        else:
            raise NotImplementedError
    return Portfolio(start_date, 1, total_cash, accounts)



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six

from rqalpha.interface import AbstractMod
from rqalpha.utils.i18n import gettext as _
from rqalpha.utils.exception import patch_user_exc
from rqalpha.const import MATCHING_TYPE

from .simulation_broker import SimulationBroker
from .signal_broker import SignalBroker
from .simulation_event_source import SimulationEventSource


class SimulationMod(AbstractMod):
    def __init__(self):
        pass

    def start_up(self, env, mod_config):
        mod_config.matching_type = self.parse_matching_type(mod_config.matching_type)
        if mod_config.commission_multiplier < 0:
            raise patch_user_exc(ValueError(_(u"invalid commission multiplier value: value range is [0, +∞)")))
        if env.config.base.margin_multiplier <= 0:
            raise patch_user_exc(ValueError(_(u"invalid margin multiplier value: value range is (0, +∞]")))

        if mod_config.signal:
            env.set_broker(SignalBroker(env, mod_config))
        else:
            env.set_broker(SimulationBroker(env, mod_config))
        event_source = SimulationEventSource(env, env.config.base.account_list)
        env.set_event_source(event_source)

    def tear_down(self, code, exception=None):
        pass

    @staticmethod
    def parse_matching_type(me_str):
        assert isinstance(me_str, six.string_types)
        if me_str == "current_bar":
            return MATCHING_TYPE.CURRENT_BAR_CLOSE
        elif me_str == "next_bar":
            return MATCHING_TYPE.NEXT_BAR_OPEN
        else:
            raise NotImplementedError



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six
import os
import ruamel.yaml as yaml
import datetime
import logbook
import locale
from pprint import pformat
import codecs
import shutil

from . import RqAttrDict, logger
from .exception import patch_user_exc
from .logger import user_log, user_system_log, system_log, std_log, user_std_handler
from ..const import RUN_TYPE, PERSIST_MODE, ACCOUNT_TYPE
from ..utils.i18n import gettext as _, localization
from ..utils.dict_func import deep_update
from ..utils.py2 import to_utf8
from ..mod.utils import mod_config_value_parse


def load_config(config_path, loader=yaml.Loader):
    if config_path is None:
        return {}
    if not os.path.exists(config_path):
        system_log.error(_(u"config.yml not found in {config_path}").format(config_path))
        return False
    with codecs.open(config_path, encoding="utf-8") as stream:
        config = yaml.load(stream, loader)
    return config


def dump_config(config_path, config, dumper=yaml.RoundTripDumper):
    with codecs.open(config_path, mode='w', encoding='utf-8') as stream:
        stream.write(to_utf8(yaml.dump(config, Dumper=dumper)))


def get_mod_config_path(generate=False):
    mod_config_path = os.path.abspath(os.path.expanduser("~/.rqalpha/mod_config.yml"))
    mod_template_path = os.path.abspath(os.path.join(os.path.dirname(os.path.realpath(__file__)), "../mod_config_template.yml"))
    if not os.path.exists(mod_config_path):
        if generate:
            dir_path = os.path.dirname(mod_config_path)
            if not os.path.exists(dir_path):
                os.makedirs(dir_path)
            shutil.copy(mod_template_path, mod_config_path)
            return mod_config_path
        else:
            return mod_template_path
    return mod_config_path


def get_user_config_path(config_path=None):
    if config_path is None:
        config_path = os.path.abspath(os.path.join(os.getcwd(), "config.yml"))
    else:
        if not os.path.exists(config_path):
            system_log.error(_("config path: {config_path} does not exist.").format(config_path=config_path))
    if not os.path.exists(config_path):
        return None
    else:
        return config_path


# def config_version_verify(config, config_path):
#     default_config_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "../default_config.yml")
#     default_config = load_config(default_config_path, verify_version=False)
#     config_version = config.get("version", None)
#     if config_version != default_config["version"]:
#         back_config_file_path = config_path + "." + datetime.datetime.now().date().strftime("%Y%m%d") + ".bak"
#         shutil.move(config_path, back_config_file_path)
#         shutil.copy(default_config_path, config_path)
#
#         system_log.warning(_(u"""
# Your current config file {config_file_path} is too old and may cause RQAlpha running error.
# RQAlpha has replaced the config file with the newest one.
# the backup config file has been saved in {back_config_file_path}.
#         """).format(config_file_path=config_path, back_config_file_path=back_config_file_path))
#         config = default_config
#     return config


def set_locale(lc):
    # FIXME: It should depends on the system and locale config
    try:
        locale.setlocale(locale.LC_ALL, "en_US.UTF-8")
        locale.setlocale(locale.LC_CTYPE, "en_US.UTF-8")
        os.environ['TZ'] = 'Asia/Shanghai'
    except Exception as e:
        if os.name != 'nt':
            raise
    localization.set_locale([lc])


def parse_config(config_args, config_path=None, click_type=True, source_code=None):
    mod_configs = config_args.pop("mod_configs", [])
    for cfg, value in mod_configs:
        key = "mod__{}".format(cfg.replace(".", "__"))
        config_args[key] = mod_config_value_parse(value)

    set_locale(config_args.get("extra__locale", None))

    user_config_path = get_user_config_path(config_path)
    mod_config_path = get_mod_config_path()

    # load default config from rqalpha
    config = load_config(os.path.join(os.path.dirname(os.path.realpath(__file__)), "../default_config.yml"))
    # load mod config
    mod_config = load_config(mod_config_path)
    deep_update(mod_config, config)
    # load user config
    user_config = load_config(user_config_path)
    deep_update(user_config, config)

    # use config_args to extend config
    if click_type:
        for key, value in six.iteritems(config_args):
            if key in ["config_path"]:
                continue
            if config_args[key] is not None:
                keys = key.split("__")
                keys.reverse()
                sub_config = config[keys.pop()]
                while True:
                    if len(keys) == 0:
                        break
                    k = keys.pop()
                    if len(keys) == 0:
                        sub_config[k] = value
                    else:
                        sub_config = sub_config[k]
    else:
        deep_update(config_args, config)

    # config from user code
    config = parse_user_config_from_code(config, source_code)
    config = RqAttrDict(config)

    base_config = config.base
    extra_config = config.extra

    set_locale(extra_config.locale)

    if isinstance(base_config.start_date, six.string_types):
        base_config.start_date = datetime.datetime.strptime(base_config.start_date, "%Y-%m-%d")
    if isinstance(base_config.start_date, datetime.datetime):
        base_config.start_date = base_config.start_date.date()
    if isinstance(base_config.end_date, six.string_types):
        base_config.end_date = datetime.datetime.strptime(base_config.end_date, "%Y-%m-%d")
    if isinstance(base_config.end_date, datetime.datetime):
        base_config.end_date = base_config.end_date.date()

    if base_config.data_bundle_path is None:
        base_config.data_bundle_path = os.path.expanduser("~/.rqalpha")

    base_config.data_bundle_path = os.path.abspath(base_config.data_bundle_path)

    if os.path.basename(base_config.data_bundle_path) != "bundle":
        base_config.data_bundle_path = os.path.join(base_config.data_bundle_path, "./bundle")

    if not os.path.exists(base_config.data_bundle_path):
        system_log.error(
            _(u"data bundle not found in {bundle_path}. Run `rqalpha update_bundle` to download data bundle.").format(
                bundle_path=base_config.data_bundle_path))
        return

    if source_code is None and not os.path.exists(base_config.strategy_file):
        system_log.error(
            _(u"strategy file not found in {strategy_file}").format(strategy_file=base_config.strategy_file))
        return

    base_config.run_type = parse_run_type(base_config.run_type)
    base_config.account_list = parse_account_list(base_config.securities)
    base_config.persist_mode = parse_persist_mode(base_config.persist_mode)

    if extra_config.log_level.upper() != "NONE":
        user_log.handlers.append(user_std_handler)
        if not extra_config.user_system_log_disabled:
            user_system_log.handlers.append(user_std_handler)

    if extra_config.context_vars:
        import simplejson as json
        if isinstance(extra_config.context_vars,  six.string_types):
            extra_config.context_vars = json.loads(to_utf8(extra_config.context_vars))

    if base_config.stock_starting_cash < 0:
        raise patch_user_exc(ValueError(_('invalid stock starting cash: {}').format(base_config.stock_starting_cash)))

    if base_config.future_starting_cash < 0:
        raise patch_user_exc(ValueError(_('invalid future starting cash: {}').format(base_config.future_starting_cash)))

    if base_config.stock_starting_cash + base_config.future_starting_cash == 0:
        raise patch_user_exc(ValueError(_('stock starting cash and future starting cash can not be both 0.')))

    system_log.level = getattr(logbook, extra_config.log_level.upper(), logbook.NOTSET)
    std_log.level = getattr(logbook, extra_config.log_level.upper(), logbook.NOTSET)
    user_log.level = getattr(logbook, extra_config.log_level.upper(), logbook.NOTSET)
    user_system_log.level = getattr(logbook, extra_config.log_level.upper(), logbook.NOTSET)

    if base_config.frequency == "1d":
        logger.DATETIME_FORMAT = "%Y-%m-%d"

    system_log.debug("\n" + pformat(config))

    return config


def parse_user_config_from_code(config, source_code=None):
    try:
        if source_code is None:
            with codecs.open(config["base"]["strategy_file"], encoding="utf-8") as f:
                source_code = f.read()

        scope = {}

        code = compile(source_code, config["base"]["strategy_file"], 'exec')
        six.exec_(code, scope)

        __config__ = scope.get("__config__", {})

        for sub_key, sub_dict in six.iteritems(__config__):
            if sub_key not in config["whitelist"]:
                continue
            deep_update(sub_dict, config[sub_key])

    except Exception as e:
        system_log.error(_('in parse_user_config, exception: {e}').format(e=e))
    finally:
        return config


def parse_account_list(securities):
    if isinstance(securities, (tuple, list)):
        return [ACCOUNT_TYPE[security.upper()] for security in securities]
    elif isinstance(securities, six.string_types):
        return [ACCOUNT_TYPE[securities.upper()]]
    else:
        raise NotImplementedError


def parse_run_type(rt_str):
    assert isinstance(rt_str, six.string_types)
    if rt_str == "b":
        return RUN_TYPE.BACKTEST
    elif rt_str == "p":
        return RUN_TYPE.PAPER_TRADING
    elif rt_str == 'r':
        return RUN_TYPE.LIVE_TRADING
    else:
        raise NotImplementedError


def parse_persist_mode(persist_mode):
    assert isinstance(persist_mode, six.string_types)
    if persist_mode == 'real_time':
        return PERSIST_MODE.REAL_TIME
    elif persist_mode == 'on_crash':
        return PERSIST_MODE.ON_CRASH
    else:
        raise RuntimeError(_('unknown persist mode: {persist_mode}').format(persist_mode=persist_mode))



#!/usr/bin/env python
# encoding: utf-8


"""
@author: Gu Xi
@contact: guxi@ricequant.com
@site: http://www.ricequant.com
@file: __init__.py.py
@time: 2017/2/17 下午1:35
"""


def func():
    pass


if __name__ == '__main__':
    pass